/*
    (C) 1995-96 AROS - The Amiga Research OS
    $Id: unloadseg.c,v 1.1.1.1 2005/03/15 15:54:40 laire Exp $

    Desc:
    Lang: english
*/
#include <proto/exec.h>
#include <dos/dos.h>
#include <exec/types.h>
#include "dos_intern.h"

#ifdef __MORPHOS__
extern const struct EmulLibEntry Exec_FreeMem;
#else
extern void Exec_FreeMem();
#endif

/*****************************************************************************

    NAME */
#include <proto/dos.h>

        AROS_LH1(LONG, UnLoadSeg,

/*  SYNOPSIS */
        AROS_LHA(BPTR, seglist, D1),

/*  LOCATION */
        struct DosLibrary *, DOSBase, 26, Dos)

/*  FUNCTION
        Free a segment list allocated with LoadSeg().

    INPUTS
        seglist - The segment list.

    RESULT
        success = returns whether everything went ok. Returns FALSE if
                  seglist was NULL.
    NOTES

    EXAMPLE

    BUGS

    SEE ALSO
        LoadSeg()

    INTERNALS

    HISTORY
        29-10-95    digulla automatically created from
                            dos_lib.fd and clib/dos_protos.h

*****************************************************************************/
{
  AROS_LIBFUNC_INIT
  AROS_LIBBASE_EXT_DECL(struct DosLibrary *,DOSBase)

  LONG success = DOSFALSE;

  DEBUG_LOADSEG(dprintf("UnLoadSeg: seg = $%lx\n", seglist));

  if (seglist)
  {
#if 1
    BPTR *p = BADDR(seglist);

    /* 17092002 Bugfix: non-overlay seglist are ok to free from task context
     */
    if (p[-1] < (8 * sizeof(BPTR)) ||
        p[2] != 0x0000ABCD || p[6] != -1 /* dl_GV */)
    {
      struct Task *me = FindTask(NULL);

      DEBUG_LOADSEG(if (me->tc_Node.ln_Type != NT_PROCESS) dprintf("UnLoadSeg: WARNING, called within task 0x%08lx \"%s\" for non-overlay seglist!\n",
                            me, me->tc_Node.ln_Name));
    }
    else
#endif
      FAILFORTASKNP(return FALSE;)

    success = InternalUnLoadSeg(seglist, (void *)&Exec_FreeMem);
  }

  return success;

  AROS_LIBFUNC_EXIT
} /* UnLoadSeg */
